import { ethers } from "https://esm.sh/ethers@6.13.1";
import { Hono } from "https://deno.land/x/hono@v3.4.1/mod.ts";
import { rateLimiter } from "https://esm.sh/hono-rate-limiter@0.3.0";

const JSON_RPC_URL = Deno.env.get('JSON_RPC_URL') // Needs to be the paid plan of merkle.io or an archival RETH/Erigon/Parity node
const FLAG = Deno.env.get('FLAG') ?? "ecsc24{fake_flag_now_try_on_the_real_thing!}";

if (JSON_RPC_URL === void 0) {
    throw new Error("JSON_RPC_URL not provided");
}
const provider = new ethers.JsonRpcProvider(JSON_RPC_URL);
const cur_chain_id = (await provider.getNetwork()).chainId;
if (cur_chain_id !== 1n) {
    throw new Error(`Expected chain id 1 got ${cur_chain_id}. The chall works on ETH mainnet`);
}

const app = new Hono()
const limiter = rateLimiter({
    windowMs: 1 * 60 * 1000, // 1 minute
    limit: 10, // 10 requests per minute
    message: "Sorry 10req/min rate limit, get your own api key :P",
    standardHeaders: "draft-6", // draft-6: `RateLimit-*` headers; draft-7: combined `RateLimit` header
    keyGenerator: (c) => c.req.header("x-forwarded-for") ?? (c.req.header("cf-connecting-ip") ?? "anon"),
});
app.get('/', (c) => c.html(`
    <html><body>
        <h1>How much money can u earn?</h1>
        <form action="/check">
            <input type="text" id="calldata" name="calldata"><br>
            <input type="submit" value="Submit">
        </form>
        </html></body>
    `))
app.use('/check', limiter as any);
app.get('/check', async (c) => {
    const body = c.req.queries("calldata");
    if (body === void 0 || body.length != 1) {
        return c.text("Nope", 400)
    }
    const user_input = body[0] as string;
    if (!/^[0-9a-fA-F]+$/.test(user_input)) {
        return c.text("Very nope", 400)
    }
    if (user_input.length % 2 == 1) {
        return c.text("Odd length hex string", 400)
    }
    const callParams = [
        {
            from: "0x45435343323032344c65657441747461636b6572",
            gas: "0x1e8480",
            data: user_input
        },
        ["stateDiff"],
        "0x1333700",
        {
            ["0xdddd770badd886df3864029e4b377b5f6a2b6b83"]: {
                code: "630902f1ac5f5260325f6004601c724375dff511095cc5a197a54140a24efef3a4165afa3d5f5f3e5f516020516024600a0a02045f5260205ff3"
            }
        }
    ];

    try {
        let r = await provider.send("trace_call", callParams);
        r = r["stateDiff"];
        if (r !== void 0) {
            r = r["0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48"];
            if (r !== void 0) {
                r = r["storage"]
                if (r !== void 0) {
                    r = r["0x20b02d88601e3871beff1058887599130cf4516fdeaac1213a2df3ea70a06c79"]
                    if (r !== void 0) {
                        r = r["*"]
                        if (r !== void 0 && r["from"] !== void 0 && r["to"] !== void 0) {
                            const f = ethers.toBigInt(r["from"]) / 1_000_000n
                            const t = ethers.toBigInt(r["to"]) / 1_000_000n
                            if (f == 0n && t >= 1_000_000n) {
                                return c.text(`Congratulations! You're a millionaire! Here is your flag:\n${FLAG}`);
                            }
                        }
                    }
                }
            }
        }
    } catch (e) {
        console.log(JSON.stringify(e));
        return c.text(`Unknown error occured. Please contact admin or try again.`)
    }

    return c.text(`Sorry, come back when you're richer :P`)
});
Deno.serve({port: 3000}, app.fetch);
